/*
 * Copyright (c) 2008 Jiri Svoboda
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup loader
 * @brief	Loads and runs programs from VFS.
 * @{
 */ 
/**
 * @file
 */

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>

#include "elf.h"

unsigned int elf_load(int fd, elf_header_t *header);

typedef void (*entry_point_t)(void);

int elf_load_file(char *file_name, elf_header_t *header)
{
	int fd;
	int rc;

	printf("open and read '%s'...\n", file_name);

	fd = open(file_name, 0);
	if (fd < 0) {
		printf("failed opening file\n");
		return -1;
	}

	rc = elf_load(fd, header);
	printf("elf_load() -> %d\n", rc);

	close(fd);

	return rc;
}

void elf_run(elf_header_t *header)
{
	entry_point_t entry_point;

	entry_point = (entry_point_t)header->e_entry;
	(*entry_point)();

	/* not reached */
}

int main(int argc, char *argv[])
{
	elf_header_t header;
	char *file_name;
	int rc;

	printf("This is loader\n");
	getchar();

	file_name = "/tetris";
	rc = elf_load_file(file_name, &header);
	printf("open and read '%s'...\n", file_name);

	if (rc < 0) {
		printf("failed to load program\n");
		return 1;
	}

	printf("run program\n");
	elf_run(&header);

	/* not reached */
	return 0;
}

/** @}
 */
